from setuptools import setup

requires = []

setup(
    name='skz_utils',
    version='0.1.2',
    description='python utils',
    url='https://github.com/skyzhao3q/skyz_pyutils.git',
    author='zhao-tian',
    author_email='skyzhaoi3q@gmail.com',
    license='MIT',
    keywords='some python utils',
    packages=[
        "skz_utils",
    ],
    install_requires=requires,
    classifiers=[
        'Programming Language :: Python :: 3.0',
    ],
)